// ---------------------------------------------------------------------
//
//     program : 2D particle moving
//
//               Takayuki Aoki
//
//               Global Scientific Information and Computing Center
//               Tokyo Institute of Technology
//
//               2009, May 1
//
// ---------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include <cutil.h>
#include <math.h>
#include "ido.h"

#define  NUM  (256*256)


int  main(int argc, char *argv[])
{
     int      np = NUM,   icnt = 1,   numGPUs,  Lx = 512,  Ly = 512,  nout = 5;
     unsigned int  timer;
     float    *x,  *y,  *xn,   *yn,   dt,   time = 0.0;
     char     filename[] = "f000.bmp";

     cudaGetDeviceCount(&numGPUs);
     numGPUs = set_numGPUs(numGPUs,argc,argv);

     malloc_variables(numGPUs,np,&x,&y,&xn,&yn);
     initial(numGPUs,np,x,y);
     r8img_bmp(numGPUs,np,x,y,Lx,Ly,filename,"Seismic.pal");

     dt = 1.0/100.0;

     cudaThreadSynchronize();

     cutCreateTimer(&timer);
     cutResetTimer(timer);
     cutStartTimer(timer);

     do {  printf("time(%4d)=%7.5f\n",icnt,time + dt);

           ppush(numGPUs,np,x,y,xn,yn,time,dt);
           swap(&x,&xn);  swap(&y, &yn);

           time += dt;

           if(icnt % nout == 0) {
              sprintf(filename,"f%03d.bmp",icnt/nout);
              r8img_bmp(numGPUs,np,x,y,Lx,Ly,filename,"Seismic.pal");
           }


     } while(icnt++ < 999999 && time < 2.0 - 0.5*dt);

     cudaThreadSynchronize();
     cutStopTimer(timer);
     float  gpu_time = cutGetTimerValue(timer)*1.0e-03;

     printf("Elapsed Time= %9.3e [sec]\n",gpu_time);

     CUT_EXIT(argc,argv);
}



void   initial
// ====================================================================
//
// purpos     :  initial positions of the particles
//
// date       :  Mar 1, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   int      numGPUs,    /* available number of GPU device            */
   int      np,         /* number of the particles                   */
   float    *x,         /* x-coordinate of the particles             */
   float    *y          /* y-coordinate of the particles             */
)
// --------------------------------------------------------------------
{
     int     j;
     float   *X,   *Y;

     if(numGPUs > 0) {
        X = (float *) malloc(np*sizeof(float));
        Y = (float *) malloc(np*sizeof(float));
     }
     else { X = x;  Y = y; }

     srand(12131);

     float  xs,   ys;
     for(j=0 ; j < np; j++) {
         do {
              xs = (float)rand()/RAND_MAX;
              ys = (float)rand()/RAND_MAX;
         } while( (xs - 0.5)*(xs - 0.5) + (ys - 0.25)*(ys - 0.25) > 0.24*0.24 );
         X[j] = xs;   Y[j] = ys;
     }

     if(numGPUs > 0) {
        cudaMemcpy(x,X,np*sizeof(float), cudaMemcpyHostToDevice);
        cudaMemcpy(y,Y,np*sizeof(float), cudaMemcpyHostToDevice);
        free(X);   free(Y);
     }
}
